﻿
/**
 * TPack 编译 ES6/JSX 为 ES5 的插件。
 * @param {BuildFile} file 当前正在生成的文件。
 * @param {Object} options 用户设置的传递给当前插件的选项。
 * @param {Builder} builder 当前正在使用的生成器。
 * @returns {String|BuildFile|Undefined} 返回新文件或新文件内容。
 */
module.exports = function (file, options, builder) {

    options.filename = file.srcPath;
    options.filenameRelative = file.srcDirPath;

    options.jsxPragma = "";
    //options.plugins = options.plugins || [];
    //options.plugins.push({
    //    transformer: htmlLikeString,
    //    position: 'after'
    //});

    // 生成源码表。
    var sourceMap = file.getSourceMapOptions(options);
    if (sourceMap) {
        //sourceMap.sourceMapFileInline = true;
        //sourceMap.sourceMapOutputFilename = sourceMap.path;
        //sourceMap.sourceMapURL = sourceMap.url || sourceMap.path;
        //sourceMap.sourceMapBasepath = options.paths[0];
        options.sourceMap = sourceMap;
    }

    // TODO: sourceMap 支持

    try {
        var result = require('babel').transform(file.content, options);
        file.content = result.code;
        file.saveSourceMap(result.map);
    } catch (e) {
        try{
            var p = /^(.*?): (.*)\s*\(\d+:\d+\)$/.exec(e.message);
            file.addError("Babel Compilation Error", e, e.__proto__.name + ": " + p[2], options.filename, e.loc.line, e.loc.column, e.codeFrame);
        } catch(e2){
            file.addError("Babel Compilation Error", e, e.toString(), null, e.loc.line, e.loc.column);
        }
    }
};

//function htmlLikeString(obj) {
//    var t = obj.types;
//    return new obj.Plugin("plugin-example", {
//        visitor: {
//            //JSXOpeningElement: function (node, parent, scope, file) {
//            //    console.log(file)
//            //},
//            //FunctionDeclaration: function (node, parent) {
//            //    var id = node.id;
//            //    node.type = "FunctionExpression";
//            //    node.id = null;

//            //    return t.variableDeclaration("var", [
//            //      t.variableDeclarator(id, node)
//            //    ]);
//            //}
//        }
//    });
//}